@@ -31,15 +31,24 @@ import (
 
 	"github.com/containerd/containerd/content"
 	"github.com/containerd/containerd/errdefs"
+	"github.com/containerd/containerd/log/logtest"
 	"github.com/containerd/containerd/pkg/testutil"
 	digest "github.com/opencontainers/go-digest"
 	ocispec "github.com/opencontainers/image-spec/specs-go/v1"
 	"github.com/pkg/errors"
-	"gotest.tools/assert"
+	"gotest.tools/v3/assert"
 )
 
+const (
+	emptyDigest = "sha256:e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855"
+)
+
+// StoreInitFn initializes content store with given root and returns a function for
+// destroying the content store
+type StoreInitFn func(ctx context.Context, root string) (context.Context, content.Store, func() error, error)
+
 // ContentSuite runs a test suite on the content store given a factory function.
-func ContentSuite(t *testing.T, name string, storeFn func(ctx context.Context, root string) (context.Context, content.Store, func() error, error)) {
+func ContentSuite(t *testing.T, name string, storeFn StoreInitFn) {
 	t.Run("Writer", makeTest(t, name, storeFn, checkContentStoreWriter))
 	t.Run("UpdateStatus", makeTest(t, name, storeFn, checkUpdateStatus))
 	t.Run("CommitExists", makeTest(t, name, storeFn, checkCommitExists))
@@ -52,10 +61,18 @@ func ContentSuite(t *testing.T, name string, storeFn func(ctx context.Context, r
 	t.Run("SmallBlob", makeTest(t, name, storeFn, checkSmallBlob))
 	t.Run("Labels", makeTest(t, name, storeFn, checkLabels))
 
+	t.Run("CommitErrorState", makeTest(t, name, storeFn, checkCommitErrorState))
+}
+
+// ContentCrossNSSharedSuite runs a test suite under shared content policy
+func ContentCrossNSSharedSuite(t *testing.T, name string, storeFn StoreInitFn) {
 	t.Run("CrossNamespaceAppend", makeTest(t, name, storeFn, checkCrossNSAppend))
 	t.Run("CrossNamespaceShare", makeTest(t, name, storeFn, checkCrossNSShare))
+}
 
-	t.Run("CommitErrorState", makeTest(t, name, storeFn, checkCommitErrorState))
+// ContentCrossNSIsolatedSuite runs a test suite under isolated content policy
+func ContentCrossNSIsolatedSuite(t *testing.T, name string, storeFn StoreInitFn) {
+	t.Run("CrossNamespaceIsolate", makeTest(t, name, storeFn, checkCrossNSIsolate))
 }
 
 // ContextWrapper is used to decorate new context used inside the test
@@ -85,6 +102,7 @@ func Name(ctx context.Context) string {
 func makeTest(t *testing.T, name string, storeFn func(ctx context.Context, root string) (context.Context, content.Store, func() error, error), fn func(ctx context.Context, t *testing.T, cs content.Store)) func(t *testing.T) {
 	return func(t *testing.T) {
 		ctx := context.WithValue(context.Background(), nameKey{}, name)
+		ctx = logtest.WithT(ctx, t)
 
 		tmpDir, err := ioutil.TempDir("", "content-suite-"+name+"-")
 		if err != nil {
@@ -322,7 +340,7 @@ func checkRefNotAvailable(ctx context.Context, t *testing.T, cs content.Store, r
 
 	w, err := cs.Writer(ctx, content.WithRef(ref))
 	if err == nil {
-		w.Close()
+		defer w.Close()
 		t.Fatal("writer created with ref, expected to be in use")
 	}
 	if !errdefs.IsUnavailable(err) {
@@ -386,6 +404,7 @@ func checkCommitErrorState(ctx context.Context, t *testing.T, cs content.Store)
 		}
 		t.Fatalf("Unexpected error: %+v", err)
 	}
+	w.Close()
 
 	w, err = cs.Writer(ctx, content.WithRef(ref))
 	if err != nil {
@@ -409,6 +428,7 @@ func checkCommitErrorState(ctx context.Context, t *testing.T, cs content.Store)
 		t.Errorf("Unexpected error: %+v", err)
 	}
 
+	w.Close()
 	w, err = cs.Writer(ctx, content.WithRef(ref))
 	if err != nil {
 		t.Fatal(err)
@@ -424,6 +444,7 @@ func checkCommitErrorState(ctx context.Context, t *testing.T, cs content.Store)
 		t.Errorf("Unexpected error: %+v", err)
 	}
 
+	w.Close()
 	w, err = cs.Writer(ctx, content.WithRef(ref))
 	if err != nil {
 		t.Fatal(err)
@@ -439,6 +460,7 @@ func checkCommitErrorState(ctx context.Context, t *testing.T, cs content.Store)
 		t.Fatalf("Failed to commit: %+v", err)
 	}
 
+	w.Close()
 	// Create another writer with same reference
 	w, err = cs.Writer(ctx, content.WithRef(ref))
 	if err != nil {
@@ -465,6 +487,7 @@ func checkCommitErrorState(ctx context.Context, t *testing.T, cs content.Store)
 		t.Fatalf("Unexpected error: %+v", err)
 	}
 
+	w.Close()
 	w, err = cs.Writer(ctx, content.WithRef(ref))
 	if err != nil {
 		t.Fatal(err)
@@ -890,6 +913,38 @@ func checkCrossNSAppend(ctx context.Context, t *testing.T, cs content.Store) {
 
 }
 
+func checkCrossNSIsolate(ctx context.Context, t *testing.T, cs content.Store) {
+	wrap, ok := ctx.Value(wrapperKey{}).(ContextWrapper)
+	if !ok {
+		t.Skip("multiple contexts not supported")
+	}
+
+	var size int64 = 1000
+	b, d := createContent(size)
+	ref := fmt.Sprintf("ref-%d", size)
+	t1 := time.Now()
+
+	if err := content.WriteBlob(ctx, cs, ref, bytes.NewReader(b), ocispec.Descriptor{Size: size, Digest: d}); err != nil {
+		t.Fatal(err)
+	}
+	t2 := time.Now()
+
+	ctx2, done, err := wrap(context.Background())
+	if err != nil {
+		t.Fatal(err)
+	}
+	defer done(ctx2)
+
+	t3 := time.Now()
+	w, err := cs.Writer(ctx2, content.WithRef(ref), content.WithDescriptor(ocispec.Descriptor{Size: size, Digest: d}))
+	if err != nil {
+		t.Fatal(err)
+	}
+	t4 := time.Now()
+
+	checkNewlyCreated(t, w, t1, t2, t3, t4)
+}
+
 func checkStatus(t *testing.T, w content.Writer, expected content.Status, d digest.Digest, preStart, postStart, preUpdate, postUpdate time.Time) {
 	t.Helper()
 	st, err := w.Status()
@@ -934,6 +989,29 @@ func checkStatus(t *testing.T, w content.Writer, expected content.Status, d dige
 	}
 }
 
+func checkNewlyCreated(t *testing.T, w content.Writer, preStart, postStart, preUpdate, postUpdate time.Time) {
+	t.Helper()
+	st, err := w.Status()
+	if err != nil {
+		t.Fatalf("failed to get status: %v", err)
+	}
+
+	wd := w.Digest()
+	if wd != emptyDigest {
+		t.Fatalf("unexpected digest %v, expected %v", wd, emptyDigest)
+	}
+
+	if st.Offset != 0 {
+		t.Fatalf("unexpected offset %v", st.Offset)
+	}
+
+	if runtime.GOOS != "windows" {
+		if st.StartedAt.After(postUpdate) || st.StartedAt.Before(postStart) {
+			t.Fatalf("unexpected started at time %s, expected between %s and %s", st.StartedAt, postStart, postUpdate)
+		}
+	}
+}
+
 func checkInfo(ctx context.Context, cs content.Store, d digest.Digest, expected content.Info, c1, c2, u1, u2 time.Time) error {
 	info, err := cs.Info(ctx, d)
 	if err != nil {
@@ -997,7 +1075,7 @@ var contentSeed int64
 
 func createContent(size int64) ([]byte, digest.Digest) {
 	// each time we call this, we want to get a different seed, but it should
-	// be related to the intitialization order and fairly consistent between
+	// be related to the initialization order and fairly consistent between
 	// test runs. An atomic integer works just good enough for this.
 	seed := atomic.AddInt64(&contentSeed, 1)
 
