@@ -167,7 +167,7 @@ func TestContainersList(t *testing.T) {
 		// try it again, get NotFound
 		if err := store.Delete(ctx, id); err == nil {
 			t.Fatalf("expected error deleting non-existent container")
-		} else if errors.Cause(err) != errdefs.ErrNotFound {
+		} else if !errdefs.IsNotFound(err) {
 			t.Fatalf("unexpected error %v", err)
 		}
 	}
@@ -636,7 +636,7 @@ func TestContainersCreateUpdateDelete(t *testing.T) {
 			now := time.Now().UTC()
 
 			result, err := store.Create(ctx, testcase.original)
-			if errors.Cause(err) != testcase.createerr {
+			if !errors.Is(err, testcase.createerr) {
 				if testcase.createerr == nil {
 					t.Fatalf("unexpected error: %v", err)
 				} else {
@@ -658,7 +658,7 @@ func TestContainersCreateUpdateDelete(t *testing.T) {
 
 			now = time.Now()
 			result, err = store.Update(ctx, testcase.input, testcase.fieldpaths...)
-			if errors.Cause(err) != testcase.cause {
+			if !errors.Is(err, testcase.cause) {
 				if testcase.cause == nil {
 					t.Fatalf("unexpected error: %v", err)
 				} else {
